import sys
import asyncio
from pathlib import Path
from playwright.async_api import async_playwright
import subprocess

HTML_TMPL = """
<!DOCTYPE html>
<html>
<head>
  <meta charset="utf-8"/>
  <script src="https://cdn.jsdelivr.net/npm/mermaid@10/dist/mermaid.min.js"></script>
  <style>
    body { margin: 0; padding: 20px; }
    #container { width: 1800px; }
    .mermaid { font-size: 16px; }
  </style>
  <script>
    mermaid.initialize({ startOnLoad: true, theme: 'default' });
  </script>
</head>
<body>
  <div id="container">
    <div class="mermaid" id="diagram">DIAGRAM_PLACEHOLDER</div>
  </div>
  </body>
</html>
"""

async def render(mermaid_text: str, out_png: Path):
    html = HTML_TMPL.replace("DIAGRAM_PLACEHOLDER", mermaid_text)
    tmp = out_png.with_suffix('.html')
    tmp.write_text(html, encoding='utf-8')
    async def _do_render():
        async with async_playwright() as p:
            browser = await p.chromium.launch()
            page = await browser.new_page(viewport={"width": 2000, "height": 1200})
            await page.goto(tmp.resolve().as_uri(), wait_until="domcontentloaded", timeout=120000)
            await page.wait_for_selector('#diagram', state='attached', timeout=120000)
            await page.evaluate("""
            try { mermaid.initialize({ startOnLoad: false, theme: 'default' }); } catch (e) {}
            """
            )
            await page.evaluate("""
            (async () => {
              try {
                await mermaid.run({ querySelector: '.mermaid' });
                const svg = document.querySelector('.mermaid svg');
                if (svg) { svg.style.width = '1800px'; svg.style.height = 'auto'; svg.style.maxWidth = 'none'; }
              } catch (e) {}
            })();
            """
            )
            await page.wait_for_selector('.mermaid svg', state='attached', timeout=120000)
            elem = await page.query_selector('#container')
            await elem.screenshot(path=str(out_png))
            await browser.close()
    try:
        await _do_render()
    except Exception:
        try:
            subprocess.run([sys.executable, '-m', 'playwright', 'install', 'chromium'], check=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
            await _do_render()
        finally:
            tmp.unlink(missing_ok=True)
    else:
        tmp.unlink(missing_ok=True)

class _BatchRenderer:
    def __init__(self, pw, browser, page):
        self._pw = pw
        self._browser = browser
        self._page = page
    @classmethod
    async def create(cls):
        pw = await async_playwright().start()
        browser = await pw.chromium.launch()
        page = await browser.new_page(viewport={"width": 2000, "height": 1200})
        return cls(pw, browser, page)
    async def render(self, mermaid_text: str, out_png: Path):
        html = HTML_TMPL.replace("DIAGRAM_PLACEHOLDER", mermaid_text)
        tmp = out_png.with_suffix('.html')
        tmp.write_text(html, encoding='utf-8')
        await self._page.goto(tmp.resolve().as_uri(), wait_until="domcontentloaded", timeout=120000)
        await self._page.wait_for_selector('#diagram', state='attached', timeout=120000)
        await self._page.evaluate("""
        try { mermaid.initialize({ startOnLoad: false, theme: 'default' }); } catch (e) {}
        """
        )
        await self._page.evaluate("""
        (async () => {
          try {
            await mermaid.run({ querySelector: '.mermaid' });
            const svg = document.querySelector('.mermaid svg');
            if (svg) { svg.style.width = '1800px'; svg.style.height = 'auto'; svg.style.maxWidth = 'none'; }
          } catch (e) {}
        })();
        """
        )
        await self._page.wait_for_selector('.mermaid svg', state='attached', timeout=120000)
        elem = await self._page.query_selector('#container')
        await elem.screenshot(path=str(out_png))
        tmp.unlink(missing_ok=True)
    async def close(self):
        try:
            await self._browser.close()
        finally:
            try:
                await self._pw.stop()
            except Exception:
                pass

_batch = None

def render_fast(mermaid_text: str, out_png: Path):
    asyncio.run(render(mermaid_text, out_png))

def close_fast():
    try:
        pass
    except Exception:
        pass

def main():
    import argparse
    parser = argparse.ArgumentParser(
        prog='HOS_M2F.2png',
        description='Mermaid 转 PNG',
        epilog=(
            '示例:\n'
            '  HOS_M2F.2png input.mmd output.png\n'
            '相关命令:\n'
            '  HOS_M2F.2docx input.md output.docx [-p paper|patent|softcopy] [-nm] [-nc] [-q]\n'
            '  HOS_M2F.2html input.md output.html [-p wx|csdn]\n'
            '  HOS_M2F.2pdf  input.md output.pdf\n'
            '  HOS_M2F.2json input.md output.json\n'
            '  HOS_M2F.2xml  input.md output.xml\n'
        ),
        formatter_class=argparse.RawTextHelpFormatter,
    )
    parser.add_argument('input', help='输入 Mermaid 文件路径')
    parser.add_argument('output', help='输出 PNG 文件路径')
    args = parser.parse_args()
    inp = Path(args.input)
    outp = Path(args.output)
    text = inp.read_text(encoding='utf-8')
    asyncio.run(render(text, outp))
    print(f'Rendered: {inp} -> {outp}')

if __name__ == '__main__':
    main()



import sys
import os
from pathlib import Path
import tempfile
from .md2docx_mermaid import parse as parse_docx

def _print_progress(name: str, percent: int, idx: int, total: int):
    w = 30
    done = int(percent * w / 100)
    bar = '#' * done + '.' * (w - done)
    msg = f"[{bar}] {percent:3d}% | {idx}/{total} | {name}"
    sys.stdout.write('\r' + msg)
    sys.stdout.flush()

def _is_md(p: Path):
    return p.suffix.lower() in ['.md', '.markdown']

def main():
    import argparse
    parser = argparse.ArgumentParser(
        prog='HOS_M2F.batch',
        description='批量将文件夹中的 Markdown 转为 DOCX（带进度）',
        epilog=(
            '示例:\n'
            '  HOS_M2F.batch input_dir output_dir [-p paper|patent|softcopy] [-nm] [-nc] [-q]\n'
            '相关命令:\n'
            '  HOS_M2F.2docx input.md output.docx [-p paper|patent|softcopy] [-nm] [-nc] [-q]\n'
            '  HOS_M2F.2html input.md output.html [-p wx|csdn]\n'
            '  HOS_M2F.2pdf  input.md output.pdf\n'
            '  HOS_M2F.2json input.md output.json\n'
            '  HOS_M2F.2xml  input.md output.xml\n'
            '  HOS_M2F.2png  input.mmd output.png\n'
        ),
    )
    parser.add_argument('input_dir', help='输入文件夹路径')
    parser.add_argument('output_dir', help='输出文件夹路径')
    parser.add_argument('-p','--profile', choices=['paper','patent','softcopy'], default='', help='DOCX 样式配置')
    parser.add_argument('-nm','--no-mermaid', action='store_true', help='禁用 Mermaid 渲染')
    parser.add_argument('-nc','--no-run-code', action='store_true', help='禁用代码执行（Matplotlib/表格）')
    parser.add_argument('-q','--quiet', action='store_true', help='静默子进程输出')
    args = parser.parse_args()

    inp = Path(args.input_dir)
    outp = Path(args.output_dir)
    outp.mkdir(parents=True, exist_ok=True)
    files = [p for p in inp.rglob('*') if p.is_file() and _is_md(p)]
    total = len(files)
    if total == 0:
        print('No markdown files found.')
        return
    for idx, f in enumerate(files, start=1):
        name = f.name
        def cb(pct):
            _print_progress(name, pct, idx, total)
        text = f.read_text(encoding='utf-8')
        with tempfile.TemporaryDirectory() as td:
            doc = parse_docx(
                text,
                Path(td),
                profile=args.profile,
                run_code=(not args.no_run_code),
                render_mermaid=(not args.no_mermaid),
                quiet=args.quiet,
                progress_cb=cb,
            )
            tgt = outp / (f.stem + '.docx')
            doc.save(str(tgt))
        _print_progress(name, 100, idx, total)
        sys.stdout.write('\n')
        sys.stdout.flush()
    print(f'Processed {total} files -> {outp}')

if __name__ == '__main__':
    main()




import sys
import tempfile
from pathlib import Path
import re
import asyncio
import json
from .render_mermaid import render as render_mermaid_png
import subprocess
import textwrap

def parse_to_model(md_text: str, workdir: Path):
    lines = md_text.splitlines()
    heading_re = re.compile(r'^(#{1,6})\s+(.*)$')
    bullet_re = re.compile(r'^\s*[-*]\s+(.*)$')
    number_re = re.compile(r'^\s*\d+\.\s+(.*)$')
    enum_re = re.compile(r'^\s*\d+[)\.、:]\s+(.*)$')
    anchor_re = re.compile(r'^<a\s+id\=\"([^\"]+)\"\s*>\s*</a>\s*$')
    table_row_re = re.compile(r'^\s*\|.*\|\s*$')
    table_sep_re = re.compile(r'^\s*\|(?:\s*:?-{3,}:?\s*\|)+\s*$')
    mermaid_inline_start_re = re.compile(r'^(?:\s*%%\{.*?\}%%\s*)?(?:graph|flowchart|sequenceDiagram|classDiagram|stateDiagram|erDiagram|gantt|journey|pie|mindmap)\b')

    def _png_size(p):
        import struct
        with open(p, 'rb') as f:
            sig = f.read(8)
            if sig != b'\x89PNG\r\n\x1a\n':
                return None
            while True:
                data = f.read(8)
                if not data:
                    return None
                clen, ctype = struct.unpack('>I4s', data)
                if ctype == b'IHDR':
                    wh = f.read(8)
                    w, h = struct.unpack('>II', wh)
                    return w, h
                f.seek(clen + 4, 1)

    blocks = []
    in_code = False
    code_lang = ''
    buf = []
    i = 0
    while i < len(lines):
        s = lines[i].rstrip('\n')
        s_norm = s.replace('\u00A0',' ').replace('\u200b','').replace('\ufeff','').replace('\u3000',' ')
        if s_norm.startswith('```'):
            if not in_code:
                in_code = True
                code_lang = s_norm[3:].strip()
                buf = []
                i += 1
                continue
            else:
                code_text = '\n'.join(buf)
                if code_lang.lower() == 'mermaid':
                    png_path = workdir / f'mmd_{i}.png'
                    try:
                        asyncio.run(render_mermaid_png(code_text, png_path))
                        blocks.append({"type":"image","path":str(png_path),"kind":"mermaid"})
                    except Exception:
                        blocks.append({"type":"code","lang":code_lang or "code","text":code_text})
                else:
                    is_matplotlib = (
                        'import matplotlib' in code_text
                        or 'from matplotlib' in code_text
                    )
                    if is_matplotlib:
                        png_path = workdir / f'pyplot_{i}.png'
                        ok = False
                        try:
                            runner = workdir / f'run_plot_{i}.py'
                            script = textwrap.dedent(f"""
                            import os
                            os.environ['MPLBACKEND'] = 'Agg'
                            import matplotlib
                            matplotlib.use('Agg')
                            import matplotlib.pyplot as plt
                            {code_text}
                            figs = [plt.figure(n) for n in plt.get_fignums()]
                            if not figs:
                                figs = [plt.gcf()]
                            figs[-1].savefig(r"{str(png_path)}", dpi=150, bbox_inches='tight')
                            plt.close('all')
                            """
                            )
                            runner.write_text(script, encoding='utf-8')
                            subprocess.run([sys.executable, str(runner)], check=True, timeout=60)
                            ok = png_path.exists()
                        except Exception:
                            ok = False
                        if ok:
                            blocks.append({"type":"image","path":str(png_path),"kind":"pyplot"})
                        else:
                            blocks.append({"type":"code","lang":code_lang or "code","text":code_text})
                    else:
                        blocks.append({"type":"code","lang":code_lang or "code","text":code_text})
                in_code = False
                code_lang = ''
                buf = []
                i += 1
                continue
        if in_code:
            buf.append(s_norm)
            i += 1
            continue
        m = heading_re.match(s)
        if m:
            lvl = len(m.group(1))
            text = m.group(2).strip()
            blocks.append({"type":"heading","level":lvl,"text":text})
            i += 1
            continue
        if s.strip()=='':
            i += 1
            continue
        ma = anchor_re.match(s)
        if ma:
            blocks.append({"type":"anchor","name":ma.group(1)})
            i += 1
            continue
        if mermaid_inline_start_re.match(s_norm):
            buf_inline = [s_norm]
            i += 1
            while i < len(lines):
                ns = lines[i].rstrip('\n')
                ns_norm = ns.replace('\u00A0',' ').replace('\u200b','').replace('\ufeff','').replace('\u3000',' ')
                if anchor_re.match(ns_norm) or heading_re.match(ns_norm) or ns_norm.startswith('```') or table_row_re.match(ns_norm):
                    break
                buf_inline.append(ns_norm)
                i += 1
            png_path = workdir / f'mmd_inline_{i}.png'
            try:
                asyncio.run(render_mermaid_png('\n'.join(buf_inline), png_path))
                blocks.append({"type":"image","path":str(png_path),"kind":"mermaid"})
            except Exception:
                blocks.append({"type":"text","text":"\n".join(buf_inline)})
            continue
        if table_row_re.match(s_norm) and (i + 1) < len(lines) and table_sep_re.match(lines[i+1].rstrip('\n').replace('\u00A0',' ').replace('\u200b','').replace('\ufeff','').replace('\u3000',' ')):
            tbl_lines = [s_norm]
            i += 1
            tbl_sep = lines[i].rstrip('\n')
            i += 1
            while i < len(lines):
                ns = lines[i].rstrip('\n')
                if table_row_re.match(ns):
                    tbl_lines.append(ns)
                    i += 1
                else:
                    break
            def parse_row(row):
                parts = [c.strip() for c in row.split('|')]
                if parts and parts[0] == '':
                    parts = parts[1:]
                if parts and parts[-1] == '':
                    parts = parts[:-1]
                return parts
            header = parse_row(tbl_lines[0])
            body_rows = [parse_row(r) for r in tbl_lines[1:]]
            png_path = workdir / f'table_{i}.png'
            ok_tbl = False
            try:
                runner = workdir / f'run_table_{i}.py'
                js_header = json.dumps(header, ensure_ascii=False)
                js_rows = json.dumps(body_rows, ensure_ascii=False)
                script = textwrap.dedent(f"""
                import os
                os.environ['MPLBACKEND'] = 'Agg'
                import matplotlib
                matplotlib.use('Agg')
                import matplotlib.pyplot as plt
                headers = {js_header}
                rows = {js_rows}
                fig, ax = plt.subplots(figsize=(max(6, len(headers)*1.2), max(2, len(rows)*0.7)))
                ax.axis('off')
                tbl = ax.table(cellText=rows, colLabels=headers, loc='center')
                tbl.auto_set_font_size(False)
                tbl.set_fontsize(9)
                tbl.scale(1, 1.25)
                fig.tight_layout()
                fig.savefig(r"{str(png_path)}", dpi=180, bbox_inches='tight')
                plt.close(fig)
                """
                )
                runner.write_text(script, encoding='utf-8')
                subprocess.run([sys.executable, str(runner)], check=True, timeout=60)
                ok_tbl = png_path.exists()
            except Exception:
                ok_tbl = False
            if ok_tbl:
                blocks.append({"type":"image","path":str(png_path),"kind":"table","meta":{"header":header,"rows":body_rows}})
            else:
                blocks.append({"type":"table","header":header,"rows":body_rows})
            continue
        mb = bullet_re.match(s)
        if mb:
            blocks.append({"type":"list","kind":"bullet","text":mb.group(1)})
            i += 1
            continue
        mn = number_re.match(s)
        if mn:
            blocks.append({"type":"list","kind":"number","text":mn.group(1)})
            i += 1
            continue
        me = enum_re.match(s)
        if me:
            blocks.append({"type":"list","kind":"enum","text":me.group(1)})
            i += 1
            continue
        blocks.append({"type":"text","text":s})
        i += 1
    return {"blocks": blocks}

def main():
    import argparse
    parser = argparse.ArgumentParser(
        prog='HOS_M2F.2json',
        description='Markdown 转 JSON 模型',
        epilog=(
            '示例:\n'
            '  HOS_M2F.2json input.md output.json\n'
            '相关命令:\n'
            '  HOS_M2F.2docx input.md output.docx [-p paper|patent|softcopy] [-nm] [-nc] [-q]\n'
            '  HOS_M2F.2html input.md output.html [-p wx|csdn]\n'
            '  HOS_M2F.2pdf  input.md output.pdf\n'
            '  HOS_M2F.2xml  input.md output.xml\n'
            '  HOS_M2F.2png  input.mmd output.png\n'
        ),
        formatter_class=argparse.RawTextHelpFormatter,
    )
    parser.add_argument('input', help='输入 Markdown 文件路径')
    parser.add_argument('output', help='输出 JSON 文件路径')
    args = parser.parse_args()
    inp = Path(args.input)
    outp = Path(args.output)
    text = inp.read_text(encoding='utf-8')
    with tempfile.TemporaryDirectory() as td:
        model = parse_to_model(text, Path(td))
        outp.write_text(json.dumps(model, ensure_ascii=False, indent=2), encoding='utf-8')
    print(f'Converted to JSON: {inp} -> {outp}')

if __name__ == '__main__':
    main()



import re
import sys
import tempfile
from pathlib import Path
from docx import Document
from docx.shared import Pt, Inches
from docx.enum.text import WD_ALIGN_PARAGRAPH
from docx.oxml import OxmlElement
from docx.oxml.ns import qn
import asyncio
from .render_mermaid import render_fast as render_mermaid_png, close_fast as close_mermaid
import subprocess
import textwrap

def sanitize(text: str) -> str:
    return ''.join(ch for ch in text if ch >= ' ' or ch in ('\t'))

def add_para(doc, text, style=None):
    p = doc.add_paragraph()
    if style:
        p.style = style
    r = p.add_run(sanitize(text))
    return p, r

def add_bookmark(paragraph, name: str, bkm_id: int):
    start = OxmlElement('w:bookmarkStart')
    start.set(qn('w:id'), str(bkm_id))
    start.set(qn('w:name'), name)
    paragraph._p.insert(0, start)
    end = OxmlElement('w:bookmarkEnd')
    end.set(qn('w:id'), str(bkm_id))
    paragraph._p.append(end)

def add_internal_link(paragraph, text: str, anchor: str):
    hyperlink = OxmlElement('w:hyperlink')
    hyperlink.set(qn('w:anchor'), anchor)
    r = OxmlElement('w:r')
    rPr = OxmlElement('w:rPr')
    rStyle = OxmlElement('w:rStyle')
    rStyle.set(qn('w:val'), 'Hyperlink')
    rPr.append(rStyle)
    t = OxmlElement('w:t')
    t.text = sanitize(text)
    r.append(rPr)
    r.append(t)
    hyperlink.append(r)
    paragraph._p.append(hyperlink)

def _apply_profile(doc: Document, profile: str):
    p = (profile or '').lower()
    normal = doc.styles['Normal']
    pf = normal.paragraph_format
    if p == 'paper':
        normal.font.size = Pt(12)
        pf.first_line_indent = Inches(0.25)
        pf.space_after = Pt(6)
        pf.space_before = Pt(0)
        for k in range(1,7):
            try:
                s = doc.styles[f'Heading {k}']
                s.font.size = Pt(max(14, 20 - k*2))
            except Exception:
                pass
    elif p == 'patent':
        normal.font.size = Pt(12)
        pf.first_line_indent = Inches(0.3)
        pf.space_after = Pt(4)
        pf.space_before = Pt(0)
        for k in range(1,7):
            try:
                s = doc.styles[f'Heading {k}']
                s.font.size = Pt(max(13, 18 - k*2))
            except Exception:
                pass
    elif p in ('softcopy','soft','software'):
        normal.font.size = Pt(12)
        pf.first_line_indent = Inches(0.25)
        pf.space_after = Pt(8)
        pf.space_before = Pt(0)
        for k in range(1,7):
            try:
                s = doc.styles[f'Heading {k}']
                s.font.size = Pt(max(14, 22 - k*2))
            except Exception:
                pass

def parse(md_text: str, workdir: Path, profile: str = '', run_code: bool = True, render_mermaid: bool = True, quiet: bool = False, progress_cb=None) -> Document:
    doc = Document()
    section = doc.sections[0]
    avail = section.page_width - section.left_margin - section.right_margin
    avail_h = section.page_height - section.top_margin - section.bottom_margin
    normal = doc.styles['Normal']
    normal.font.size = Pt(11)
    pf = normal.paragraph_format
    pf.space_after = Pt(6)
    pf.space_before = Pt(0)
    _apply_profile(doc, profile)
    lines = md_text.splitlines()
    pre_heading_re = re.compile(r'^(#{1,6})\s+(.*)$')
    pre_anchor_re = re.compile(r'^<a\s+id\=\"([^\"]+)\"\s*>\s*</a>\s*$')
    pre_toclink_re = re.compile(r'^\s*[-*]\s*\[(.*?)\]\(#([^\)]+)\)\s*$')
    toc_items = []
    pending_anchor = None
    def strip_leading_number(t: str) -> str:
        return re.sub(r'^\s*\d+(?:\.\d+)*\s+', '', t)
    pre_counters = [0,0,0,0,0,0]
    pre_in_code = False
    toc_anchor_set = set()
    in_toc_scan = False
    j = 0
    while j < len(lines):
        ln = lines[j]
        s0 = ln.rstrip('\n')
        if s0.startswith('## 目录'):
            in_toc_scan = True
            j += 1
            continue
        if in_toc_scan:
            mt0 = pre_toclink_re.match(s0)
            if mt0:
                toc_anchor_set.add(mt0.group(2))
                j += 1
                continue
            else:
                in_toc_scan = False
        if s0.startswith('```'):
            pre_in_code = not pre_in_code
            j += 1
            continue
        if pre_in_code:
            j += 1
            continue
        ms = pre_anchor_re.match(s0)
        if ms:
            pending_anchor = ms.group(1)
            j += 1
            continue
        mh = pre_heading_re.match(s0)
        if mh:
            lvl = len(mh.group(1))
            txt = strip_leading_number(mh.group(2).strip())
            pre_counters[lvl-1] += 1
            for k in range(lvl, 6):
                pre_counters[k] = 0
            num = '.'.join(str(pre_counters[x]) for x in range(lvl) if pre_counters[x] > 0)
            toc_items.append((lvl, f"{num} {txt}", pending_anchor))
            pending_anchor = None
        j += 1
    in_code = False
    code_lang = ''
    buf = []
    heading_re = re.compile(r'^(#{1,6})\s+(.*)$')
    bullet_re = re.compile(r'^\s*[-*]\s+(.*)$')
    number_re = re.compile(r'^\s*\d+\.\s+(.*)$')
    enum_re = re.compile(r'^\s*\d+[)\.、:]\s+(.*)$')
    anchor_re = re.compile(r'^<a\s+id\=\"([^\"]+)\"\s*>\s*</a>\s*$')
    mdlink_re = re.compile(r'^\s*\[(.*?)\]\(#([^\)]+)\)\s*$')
    toclink_re = re.compile(r'^\s*[-*]\s*\[(.*?)\]\(#([^\)]+)\)\s*$')
    table_row_re = re.compile(r'^\s*\|.*\|\s*$')
    table_sep_re = re.compile(r'^\s*\|(?:\s*:?-{3,}:?\s*\|)+\s*$')
    mermaid_inline_start_re = re.compile(r'^(?:\s*%%\{.*?\}%%\s*)?(?:graph|flowchart|sequenceDiagram|classDiagram|stateDiagram|erDiagram|gantt|journey|pie|mindmap)\b')

    bkm_counter = 1
    in_toc = False
    in_html_comment = False
    prev_blank = False
    counters = [0,0,0,0,0,0]
    pending_anchor_name = None

    i = 0
    while i < len(lines):
        line = lines[i]
        s = line.rstrip('\n')
        s_norm = s.replace('\u00A0',' ').replace('\u200b','').replace('\ufeff','').replace('\u3000',' ')
        if not in_code:
            if '<!--' in s and '-->' not in s:
                in_html_comment = True
                i += 1
                continue
            if in_html_comment:
                if '-->' in s:
                    s = s.split('-->',1)[1].strip()
                    in_html_comment = False
                else:
                    i += 1
                    continue
            s = re.sub(r'<!--.*?-->', '', s)
            if re.match(r'^\s*-{3,}\s*$', s):
                i += 1
                continue
        if s_norm.startswith('```'):
            if not in_code:
                in_code = True
                code_lang = s_norm[3:].strip()
                buf = []
            else:
                if code_lang.lower() == 'mermaid':
                    mmd_path = workdir / f'mmd_{i}.mmd'
                    png_path = workdir / f'mmd_{i}.png'
                    mmd_path.write_text('\n'.join(buf), encoding='utf-8')
                    try:
                        if not render_mermaid:
                            raise BaseException('mermaid rendering disabled')
                        render_mermaid_png('\n'.join(buf), png_path)
                        def _png_size(p):
                            import struct
                            with open(p, 'rb') as f:
                                sig = f.read(8)
                                if sig != b'\x89PNG\r\n\x1a\n':
                                    return None
                                while True:
                                    data = f.read(8)
                                    if not data:
                                        return None
                                    clen, ctype = struct.unpack('>I4s', data)
                                    if ctype == b'IHDR':
                                        wh = f.read(8)
                                        w, h = struct.unpack('>II', wh)
                                        return w, h
                                    f.seek(clen + 4, 1)
                        sz = _png_size(str(png_path))
                        pic = doc.add_picture(str(png_path))
                        if sz:
                            w_px, h_px = sz
                            max_w = avail
                            max_h = int(avail_h * 3 / 5)
                            if w_px == 0 or h_px == 0:
                                scaled_w, scaled_h = max_w, int(max_h)
                            else:
                                ratio = w_px / h_px
                                scaled_w = max_w
                                scaled_h = int(max_w / ratio)
                                if scaled_h > max_h:
                                    scaled_h = int(max_h)
                                    scaled_w = int(max_h * ratio)
                            pic.width = scaled_w
                            pic.height = scaled_h
                        if doc.paragraphs:
                            doc.paragraphs[-1].alignment = WD_ALIGN_PARAGRAPH.CENTER
                    except BaseException:
                        add_para(doc, '[mermaid]', None)
                        for l in buf:
                            p, r = add_para(doc, l, None)
                            r.font.name = 'Consolas'
                            r.font.size = Pt(10)
                else:
                    code_text = '\n'.join(buf)
                    code_text_clean = (
                        code_text.replace('\u00A0',' ').replace('\u200b','').replace('\ufeff','').replace('\u3000',' ')
                    )
                    is_matplotlib = (
                        'import matplotlib' in code_text
                        or 'from matplotlib' in code_text
                    )
                    if is_matplotlib and run_code:
                        png_path = workdir / f'pyplot_{i}.png'
                        ok = False
                        try:
                            runner = workdir / f'run_plot_{i}.py'
                            script_lines = [
                                "import os",
                                "os.environ['MPLBACKEND'] = 'Agg'",
                                "import matplotlib",
                                "matplotlib.use('Agg')",
                                "import matplotlib.pyplot as plt",
                                "from matplotlib import font_manager",
                                "_cjk_candidates = ['Microsoft YaHei','SimHei','Noto Sans CJK SC','Source Han Sans SC','Arial Unicode MS']",
                                "for _name in _cjk_candidates:",
                                "    try:",
                                "        _path = font_manager.findfont(_name, fallback_to_default=False)",
                                "        if _path:",
                                "            matplotlib.rcParams['font.sans-serif'] = [_name]",
                                "            matplotlib.rcParams['axes.unicode_minus'] = False",
                                "            break",
                                "    except Exception:",
                                "        pass",
                                "",
                                "# === user code start ===",
                                code_text_clean,
                                "# === user code end ===",
                                "try:",
                                "    figs = [plt.figure(n) for n in plt.get_fignums()]",
                                "    if not figs:",
                                "        figs = [plt.gcf()]",
                                f"    figs[-1].savefig(r\"{str(png_path)}\", dpi=150, bbox_inches='tight')",
                                "except Exception as _e:",
                                "    raise",
                                "finally:",
                                "    plt.close('all')",
                            ]
                            runner.write_text("\n".join(script_lines), encoding='utf-8')
                            if quiet:
                                subprocess.run([sys.executable, str(runner)], check=True, timeout=60, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
                            else:
                                subprocess.run([sys.executable, str(runner)], check=True, timeout=60)
                            ok = png_path.exists()
                        except BaseException:
                            ok = False
                        if ok:
                            def _png_size(p):
                                import struct
                                with open(p, 'rb') as f:
                                    sig = f.read(8)
                                    if sig != b'\x89PNG\r\n\x1a\n':
                                        return None
                                    while True:
                                        data = f.read(8)
                                        if not data:
                                            return None
                                        clen, ctype = struct.unpack('>I4s', data)
                                        if ctype == b'IHDR':
                                            wh = f.read(8)
                                            w, h = struct.unpack('>II', wh)
                                            return w, h
                                        f.seek(clen + 4, 1)
                            sz = _png_size(str(png_path))
                            pic = doc.add_picture(str(png_path))
                            if sz:
                                w_px, h_px = sz
                                max_w = avail
                                max_h = int(avail_h * 3 / 5)
                                if w_px == 0 or h_px == 0:
                                    scaled_w, scaled_h = max_w, int(max_h)
                                else:
                                    ratio = w_px / h_px
                                    scaled_w = max_w
                                    scaled_h = int(max_w / ratio)
                                    if scaled_h > max_h:
                                        scaled_h = int(max_h)
                                        scaled_w = int(max_h * ratio)
                                pic.width = scaled_w
                                pic.height = scaled_h
                            if doc.paragraphs:
                                doc.paragraphs[-1].alignment = WD_ALIGN_PARAGRAPH.CENTER
                        else:
                            add_para(doc, f'[{code_lang or "code"}]', None)
                            for l in buf:
                                p, r = add_para(doc, l, None)
                                r.font.name = 'Consolas'
                                r.font.size = Pt(10)
                    else:
                        add_para(doc, f'[{code_lang or "code"}]', None)
                        for l in buf:
                            p, r = add_para(doc, l, None)
                            r.font.name = 'Consolas'
                            r.font.size = Pt(10)
                in_code = False
                code_lang = ''
                buf = []
            i += 1
            try:
                if progress_cb:
                    progress_cb(min(99, int(i * 100 / max(1, len(lines)))))
            except Exception:
                pass
            continue
        if in_code:
            buf.append(s_norm)
            i += 1
            try:
                if progress_cb:
                    progress_cb(min(99, int(i * 100 / max(1, len(lines)))))
            except Exception:
                pass
            continue

        if s_norm.startswith('## 目录'):
            p, r = add_para(doc, '目录', 'Heading 2')
            r.font.size = Pt(20)
            p.alignment = WD_ALIGN_PARAGRAPH.CENTER
            in_toc = True
            i += 1
            try:
                if progress_cb:
                    progress_cb(min(99, int(i * 100 / max(1, len(lines)))))
            except Exception:
                pass
            continue

        if in_toc:
            if s.strip() == '':
                doc.add_page_break()
                in_toc = False
                i += 1
                continue
            mt = toclink_re.match(s)
            if mt:
                txt, anc = mt.group(1), mt.group(2)
                mlead = re.match(r'^(\s*)[-*]', s)
                indent_spaces = len(mlead.group(1)) if mlead else 0
                indent_level = max(0, indent_spaces // 2)
                tp = doc.add_paragraph()
                tp.paragraph_format.left_indent = Inches(0.25) * indent_level
                add_internal_link(tp, txt, anc)
                i += 1
                continue
            doc.add_page_break()
            in_toc = False

        # Marker-based inline mermaid: a line with [mermaid] followed by diagram lines
        if not in_code and s_norm.strip().lower() == '[mermaid]':
            buf_inline = []
            i += 1
            while i < len(lines):
                ns = lines[i].rstrip('\n')
                ns_norm = ns.replace('\u00A0',' ').replace('\u200b','').replace('\ufeff','').replace('\u3000',' ')
                if anchor_re.match(ns_norm) or heading_re.match(ns_norm) or ns_norm.strip() == '' or ns_norm.startswith('```') or table_row_re.match(ns_norm):
                    break
                buf_inline.append(ns_norm)
                i += 1
            mmd_path = workdir / f'mmd_inline_{i}.mmd'
            png_path = workdir / f'mmd_inline_{i}.png'
            mmd_path.write_text('\n'.join(buf_inline), encoding='utf-8')
            try:
                if not render_mermaid:
                    raise BaseException('mermaid rendering disabled')
                render_mermaid_png('\n'.join(buf_inline), png_path)
                def _png_size(p):
                    import struct
                    with open(p, 'rb') as f:
                        sig = f.read(8)
                        if sig != b'\x89PNG\r\n\x1a\n':
                            return None
                        while True:
                            data = f.read(8)
                            if not data:
                                return None
                            clen, ctype = struct.unpack('>I4s', data)
                            if ctype == b'IHDR':
                                wh = f.read(8)
                                w, h = struct.unpack('>II', wh)
                                return w, h
                            f.seek(clen + 4, 1)
                sz = _png_size(str(png_path))
                pic = doc.add_picture(str(png_path))
                if sz:
                    w_px, h_px = sz
                    max_w = avail
                    max_h = int(avail_h * 3 / 5)
                    if w_px == 0 or h_px == 0:
                        scaled_w, scaled_h = max_w, int(max_h)
                    else:
                        ratio = w_px / h_px
                        scaled_w = max_w
                        scaled_h = int(max_w / ratio)
                        if scaled_h > max_h:
                            scaled_h = int(max_h)
                            scaled_w = int(max_h * ratio)
                    pic.width = scaled_w
                    pic.height = scaled_h
                if doc.paragraphs:
                    doc.paragraphs[-1].alignment = WD_ALIGN_PARAGRAPH.CENTER
            except BaseException:
                add_para(doc, '[mermaid]', None)
                for l in buf_inline:
                    p, r = add_para(doc, l, None)
                    r.font.name = 'Consolas'
                    r.font.size = Pt(10)
            i += 1
            try:
                if progress_cb:
                    progress_cb(min(99, int(i * 100 / max(1, len(lines)))))
            except Exception:
                pass
            continue

        if not in_code and mermaid_inline_start_re.match(s_norm):
            buf_inline = [s]
            i += 1
            while i < len(lines):
                ns = lines[i].rstrip('\n')
                ns_norm = ns.replace('\u00A0',' ').replace('\u200b','').replace('\ufeff','').replace('\u3000',' ')
                if anchor_re.match(ns_norm) or heading_re.match(ns_norm) or ns_norm.startswith('```') or table_row_re.match(ns_norm):
                    break
                buf_inline.append(ns_norm)
                i += 1
            mmd_path = workdir / f'mmd_inline_{i}.mmd'
            png_path = workdir / f'mmd_inline_{i}.png'
            mmd_path.write_text('\n'.join(buf_inline), encoding='utf-8')
            try:
                if not render_mermaid:
                    raise BaseException('mermaid rendering disabled')
                asyncio.run(render_mermaid_png('\n'.join(buf_inline), png_path))
                def _png_size(p):
                    import struct
                    with open(p, 'rb') as f:
                        sig = f.read(8)
                        if sig != b'\x89PNG\r\n\x1a\n':
                            return None
                        while True:
                            data = f.read(8)
                            if not data:
                                return None
                            clen, ctype = struct.unpack('>I4s', data)
                            if ctype == b'IHDR':
                                wh = f.read(8)
                                w, h = struct.unpack('>II', wh)
                                return w, h
                            f.seek(clen + 4, 1)
                sz = _png_size(str(png_path))
                pic = doc.add_picture(str(png_path))
                if sz:
                    w_px, h_px = sz
                    max_w = avail
                    max_h = int(avail_h * 3 / 5)
                    if w_px == 0 or h_px == 0:
                        scaled_w, scaled_h = max_w, int(max_h)
                    else:
                        ratio = w_px / h_px
                        scaled_w = max_w
                        scaled_h = int(max_w / ratio)
                        if scaled_h > max_h:
                            scaled_h = int(max_h)
                            scaled_w = int(max_h * ratio)
                    pic.width = scaled_w
                    pic.height = scaled_h
                if doc.paragraphs:
                    doc.paragraphs[-1].alignment = WD_ALIGN_PARAGRAPH.CENTER
            except BaseException:
                for l in buf_inline:
                    p, r = add_para(doc, l, None)
                    r.font.name = 'Consolas'
                    r.font.size = Pt(10)
            continue

        if table_row_re.match(s) and (i + 1) < len(lines) and table_sep_re.match(lines[i+1].rstrip('\n')):
            tbl_lines = [s]
            i += 1
            tbl_sep = lines[i].rstrip('\n')
            i += 1
            while i < len(lines):
                ns = lines[i].rstrip('\n')
                if table_row_re.match(ns):
                    tbl_lines.append(ns)
                    i += 1
                else:
                    break

            def parse_row(row):
                parts = [c.strip() for c in row.split('|')]
                if parts and parts[0] == '':
                    parts = parts[1:]
                if parts and parts[-1] == '':
                    parts = parts[:-1]
                return parts

            header = parse_row(tbl_lines[0])
            sep = parse_row(tbl_sep)
            body_rows = [parse_row(r) for r in tbl_lines[1:]]

            png_path = workdir / f'table_{i}.png'
            ok_tbl = False
            try:
                runner = workdir / f'run_table_{i}.py'
                import json
                js_header = json.dumps(header, ensure_ascii=False)
                js_rows = json.dumps(body_rows, ensure_ascii=False)
                script_lines = [
                    "import os",
                    "os.environ['MPLBACKEND'] = 'Agg'",
                    "import matplotlib",
                    "matplotlib.use('Agg')",
                    "import matplotlib.pyplot as plt",
                    "from matplotlib import font_manager",
                    "from matplotlib.font_manager import FontProperties",
                    "_cjk_candidates = ['Microsoft YaHei','SimHei','Noto Sans CJK SC','Source Han Sans SC','Arial Unicode MS']",
                    "for _name in _cjk_candidates:",
                    "    try:",
                    "        _path = font_manager.findfont(_name, fallback_to_default=False)",
                    "        if _path:",
                    "            matplotlib.rcParams['font.sans-serif'] = [_name]",
                    "            matplotlib.rcParams['axes.unicode_minus'] = False",
                    "            break",
                    "    except Exception:",
                    "        pass",
                    f"headers = {js_header}",
                    f"rows = {js_rows}",
                    "fig, ax = plt.subplots(figsize=(max(6, len(headers)*1.2), max(2, len(rows)*0.7)))",
                    "ax.axis('off')",
                    "tbl = ax.table(cellText=rows, colLabels=headers, loc='center')",
                    "tbl.auto_set_font_size(False)",
                    "tbl.set_fontsize(9)",
                    "_fam = matplotlib.rcParams.get('font.sans-serif',[None])[0]",
                    "_fp = FontProperties(family=_fam, size=9) if _fam else None",
                    "if _fp is not None:",
                    "    for _c in tbl.get_celld().values():",
                    "        _c.get_text().set_fontproperties(_fp)",
                    "tbl.scale(1, 1.25)",
                    "fig.tight_layout()",
                    f"fig.savefig(r\"{str(png_path)}\", dpi=180, bbox_inches='tight')",
                    "plt.close(fig)",
                ]
                runner.write_text("\n".join(script_lines), encoding='utf-8')
                if run_code:
                    if quiet:
                        subprocess.run([sys.executable, str(runner)], check=True, timeout=60, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
                    else:
                        subprocess.run([sys.executable, str(runner)], check=True, timeout=60)
                ok_tbl = png_path.exists()
            except Exception:
                ok_tbl = False

            if ok_tbl:
                def _png_size(p):
                    import struct
                    with open(p, 'rb') as f:
                        sig = f.read(8)
                        if sig != b'\x89PNG\r\n\x1a\n':
                            return None
                        while True:
                            data = f.read(8)
                            if not data:
                                return None
                            clen, ctype = struct.unpack('>I4s', data)
                            if ctype == b'IHDR':
                                wh = f.read(8)
                                w, h = struct.unpack('>II', wh)
                                return w, h
                            f.seek(clen + 4, 1)
                sz = _png_size(str(png_path))
                pic = doc.add_picture(str(png_path))
                if sz:
                    w_px, h_px = sz
                    max_w = avail
                    max_h = int(avail_h * 3 / 5)
                    if w_px == 0 or h_px == 0:
                        scaled_w, scaled_h = max_w, int(max_h)
                    else:
                        ratio = w_px / h_px
                        scaled_w = max_w
                        scaled_h = int(max_w / ratio)
                        if scaled_h > max_h:
                            scaled_h = int(max_h)
                            scaled_w = int(max_h * ratio)
                    pic.width = scaled_w
                    pic.height = scaled_h
                if doc.paragraphs:
                    doc.paragraphs[-1].alignment = WD_ALIGN_PARAGRAPH.CENTER
                continue
            else:
                aligns = []
                for p in sep:
                    ps = p.strip()
                    left = ps.startswith(':')
                    right = ps.endswith(':')
                    if left and right:
                        aligns.append(WD_ALIGN_PARAGRAPH.CENTER)
                    elif right:
                        aligns.append(WD_ALIGN_PARAGRAPH.RIGHT)
                    else:
                        aligns.append(WD_ALIGN_PARAGRAPH.LEFT)
                cols = len(header)
                rows = len(body_rows) + 1
                table = doc.add_table(rows=rows, cols=cols)
                table.style = 'Table Grid'
                for j, h in enumerate(header):
                    cell = table.cell(0, j)
                    cell.text = ''
                    para = cell.paragraphs[0]
                    para.alignment = aligns[j] if j < len(aligns) else WD_ALIGN_PARAGRAPH.LEFT
                    run = para.add_run(sanitize(h))
                    run.bold = True
                for ri, row_vals in enumerate(body_rows):
                    for cj in range(cols):
                        val = row_vals[cj] if cj < len(row_vals) else ''
                        cell = table.cell(ri + 1, cj)
                        cell.text = ''
                        para = cell.paragraphs[0]
                        para.alignment = aligns[cj] if cj < len(aligns) else WD_ALIGN_PARAGRAPH.LEFT
                        para.add_run(sanitize(val))
                continue

        ma = anchor_re.match(s)
        if ma:
            name = ma.group(1)
            pending_anchor_name = name if name in toc_anchor_set else None
            i += 1
            try:
                if progress_cb:
                    progress_cb(min(99, int(i * 100 / max(1, len(lines)))))
            except Exception:
                pass
            continue

        ml = mdlink_re.match(s)
        if ml:
            text = ml.group(1)
            anchor = ml.group(2)
            p = doc.add_paragraph()
            add_internal_link(p, text, anchor)
            i += 1
            continue

        m = heading_re.match(s)
        if m:
            lvl = len(m.group(1))
            text = m.group(2).strip()
            style = f'Heading {min(6,lvl)}'
            p, r = add_para(doc, text, style)
            if pending_anchor_name:
                add_bookmark(p, pending_anchor_name, bkm_counter)
                bkm_counter += 1
                pending_anchor_name = None
            i += 1
            continue
        mb = bullet_re.match(s)
        if mb:
            add_para(doc, mb.group(1), None)
            i += 1
            continue
        mn = number_re.match(s)
        if mn:
            add_para(doc, mn.group(1), None)
            i += 1
            continue
        me = enum_re.match(s)
        if me:
            add_para(doc, me.group(1), None)
            i += 1
            continue
        if s.strip() == '':
            if not prev_blank:
                doc.add_paragraph('')
                prev_blank = True
            i += 1
            continue
        else:
            prev_blank = False
        add_para(doc, s, None)
        i += 1
    return doc

def main():
    import argparse
    import warnings
    warnings.filterwarnings("ignore", category=ResourceWarning)
    parser = argparse.ArgumentParser(
        prog='HOS_M2F.2docx',
        description='Markdown 转 DOCX，支持 Mermaid 渲染与 Matplotlib/表格转换为图片',
        epilog=(
            '示例:\n'
            '  HOS_M2F.2docx input.md output.docx [-p paper|patent|softcopy] [-nm] [-nc] [-q]\n'
            '相关命令:\n'
            '  HOS_M2F.2html input.md output.html [-p wx|csdn]\n'
            '  HOS_M2F.2pdf  input.md output.pdf\n'
            '  HOS_M2F.2json input.md output.json\n'
            '  HOS_M2F.2xml  input.md output.xml\n'
            '  HOS_M2F.2png  input.mmd output.png\n'
        ),
        formatter_class=argparse.RawTextHelpFormatter,
    )
    parser.add_argument('input', help='输入 Markdown 文件路径')
    parser.add_argument('output', help='输出 DOCX 文件路径')
    parser.add_argument('-p','--profile', choices=['paper','patent','softcopy'], default='', help='DOCX 段落/标题样式配置')
    parser.add_argument('-nm','--no-mermaid', action='store_true', help='禁用 Mermaid 渲染，按文本插入')
    parser.add_argument('-nc','--no-run-code', action='store_true', help='禁用代码执行（Matplotlib/表格渲染），按文本或表格插入')
    parser.add_argument('-q','--quiet', action='store_true', help='静默子进程输出')
    args = parser.parse_args()

    inp = Path(args.input)
    outp = Path(args.output)
    text = inp.read_text(encoding='utf-8')
    with tempfile.TemporaryDirectory() as td:
        doc = parse(
            text,
            Path(td),
            profile=args.profile,
            run_code=(not args.no_run_code),
            render_mermaid=(not args.no_mermaid),
            quiet=args.quiet,
        )
        doc.save(str(outp))
    try:
        close_mermaid()
    except Exception:
        pass
    print(f'Converted with mermaid: {inp} -> {outp}')

if __name__ == '__main__':
    main()



def main():
    print('可用命令:')
    print('  HOS_M2F.2docx input.md output.docx [-p PROFILE] [-nm] [-nc] [-q]')
    print('    -p, --profile {paper,patent,softcopy}  设置 DOCX 样式')
    print('    -nm, --no-mermaid                      禁用 Mermaid 渲染，按文本插入')
    print('    -nc, --no-run-code                     禁用代码执行（Matplotlib/表格），按文本/表格插入')
    print('    -q,  --quiet                           静默子进程输出')
    print('  HOS_M2F.2html  input.md output.html [-p PROFILE]')
    print('    -p, --profile {wx,csdn}                设置博客样式')
    print('  HOS_M2F.2pdf   input.md output.pdf')
    print('  HOS_M2F.2json  input.md output.json')
    print('  HOS_M2F.2xml   input.md output.xml')
    print('  HOS_M2F.2png   input.mmd output.png')
    print('  HOS_M2F.batch  input_dir output_dir [-p PROFILE] [-nm] [-nc] [-q]')
    print('示例:')
    print('  HOS_M2F.2docx in.md out.docx -p paper -q')
    print('  HOS_M2F.2docx in.md out.docx -nm -nc')
    print('  HOS_M2F.2html in.md out.html -p wx')
    print('  HOS_M2F.batch in_dir out_dir -p paper -q')

if __name__ == '__main__':
    main()



import sys
import tempfile
from pathlib import Path
import xml.etree.ElementTree as ET
from .md2json import parse_to_model

def main():
    import argparse
    parser = argparse.ArgumentParser(
        prog='HOS_M2F.2xml',
        description='Markdown 转 XML',
        epilog=(
            '示例:\n'
            '  HOS_M2F.2xml input.md output.xml\n'
            '相关命令:\n'
            '  HOS_M2F.2docx input.md output.docx [-p paper|patent|softcopy] [-nm] [-nc] [-q]\n'
            '  HOS_M2F.2html input.md output.html [-p wx|csdn]\n'
            '  HOS_M2F.2pdf  input.md output.pdf\n'
            '  HOS_M2F.2json input.md output.json\n'
            '  HOS_M2F.2png  input.mmd output.png\n'
        ),
        formatter_class=argparse.RawTextHelpFormatter,
    )
    parser.add_argument('input', help='输入 Markdown 文件路径')
    parser.add_argument('output', help='输出 XML 文件路径')
    args = parser.parse_args()
    inp = Path(args.input)
    outp = Path(args.output)
    text = inp.read_text(encoding='utf-8')
    with tempfile.TemporaryDirectory() as td:
        model = parse_to_model(text, Path(td))
        root = ET.Element('document')
        for b in model['blocks']:
            t = b['type']
            if t == 'heading':
                e = ET.SubElement(root, 'heading')
                e.set('level', str(b.get('level', 1)))
                e.text = b.get('text', '')
            elif t == 'text':
                e = ET.SubElement(root, 'paragraph')
                e.text = b.get('text', '')
            elif t == 'code':
                e = ET.SubElement(root, 'code')
                e.set('lang', b.get('lang',''))
                e.text = b.get('text','')
            elif t == 'image':
                e = ET.SubElement(root, 'image')
                e.set('kind', b.get('kind',''))
                e.set('path', b.get('path',''))
            elif t == 'table':
                e = ET.SubElement(root, 'table')
                h = ET.SubElement(e, 'header')
                for c in b.get('header',[]):
                    ET.SubElement(h,'cell').text = c
                body = ET.SubElement(e,'body')
                for r in b.get('rows',[]):
                    row = ET.SubElement(body,'row')
                    for c in r:
                        ET.SubElement(row,'cell').text = c
            elif t == 'anchor':
                e = ET.SubElement(root, 'anchor')
                e.set('name', b.get('name',''))
            elif t == 'list':
                e = ET.SubElement(root, 'list')
                e.set('kind', b.get('kind',''))
                e.text = b.get('text','')
        tree = ET.ElementTree(root)
        ET.indent(tree, space='  ')
        tree.write(str(outp), encoding='utf-8', xml_declaration=True)
    print(f'Converted to XML: {inp} -> {outp}')

if __name__ == '__main__':
    main()



import sys
import tempfile
from pathlib import Path
from .md2json import parse_to_model

def _css(profile: str):
    p = (profile or '').lower()
    base = "body{margin:16px;font-size:16px;line-height:1.75;} img{max-width:100%;height:auto;} pre{background:#f6f6f6;padding:12px;overflow:auto;} code{font-family:Consolas,monospace;} h1,h2,h3{margin:1em 0 0.5em;} ul,ol{margin:0.5em 0 0.5em 1.5em;}"
    if p == 'wx':
        return base + " p{word-break:break-word;}"
    if p == 'csdn':
        return base + " pre{background:#2b2b2b;color:#ddd;}"
    return base

def render_html(model, profile: str):
    css = _css(profile)
    out = ["<html><head><meta charset=\"utf-8\"><style>" + css + "</style></head><body>"]
    for b in model['blocks']:
        t = b['type']
        if t == 'heading':
            lvl = int(b.get('level',1))
            lvl = 1 if lvl < 1 else 6 if lvl > 6 else lvl
            out.append(f"<h{lvl}>" + b.get('text','') + f"</h{lvl}>")
        elif t == 'text':
            out.append("<p>" + b.get('text','') + "</p>")
        elif t == 'list':
            out.append("<p>• " + b.get('text','') + "</p>")
        elif t == 'code':
            s = b.get('text','').replace('&','&amp;').replace('<','&lt;').replace('>','&gt;')
            out.append("<pre><code>" + s + "</code></pre>")
        elif t == 'image':
            p = b.get('path','')
            if p:
                out.append(f"<p><img src=\"{p}\"/></p>")
        elif t == 'table':
            hdr = b.get('header',[])
            rows = b.get('rows',[])
            out.append("<table>")
            if hdr:
                out.append("<thead><tr>" + ''.join([f"<th>{c}</th>" for c in hdr]) + "</tr></thead>")
            for r in rows:
                out.append("<tr>" + ''.join([f"<td>{c}</td>" for c in r]) + "</tr>")
            out.append("</table>")
    out.append("</body></html>")
    return "".join(out)

def main():
    import argparse
    parser = argparse.ArgumentParser(
        prog='HOS_M2F.2html',
        description='Markdown 转 HTML，支持博客样式',
        epilog=(
            '示例:\n'
            '  HOS_M2F.2html input.md output.html [-p wx|csdn]\n'
            '相关命令:\n'
            '  HOS_M2F.2docx input.md output.docx [-p paper|patent|softcopy] [-nm] [-nc] [-q]\n'
            '  HOS_M2F.2pdf  input.md output.pdf\n'
            '  HOS_M2F.2json input.md output.json\n'
            '  HOS_M2F.2xml  input.md output.xml\n'
            '  HOS_M2F.2png  input.mmd output.png\n'
        ),
        formatter_class=argparse.RawTextHelpFormatter,
    )
    parser.add_argument('input', help='输入 Markdown 文件路径')
    parser.add_argument('output', help='输出 HTML 文件路径')
    parser.add_argument('-p','--profile', choices=['wx','csdn'], default='', help='博客样式配置')
    args = parser.parse_args()
    inp = Path(args.input)
    outp = Path(args.output)
    text = inp.read_text(encoding='utf-8')
    with tempfile.TemporaryDirectory() as td:
        model = parse_to_model(text, Path(td))
        html = render_html(model, args.profile)
        outp.write_text(html, encoding='utf-8')
    print(f'Converted to HTML: {inp} -> {outp}')

if __name__ == '__main__':
    main()



import sys
import tempfile
from pathlib import Path
from .md2json import parse_to_model
from reportlab.platypus import SimpleDocTemplate, Paragraph, Spacer, Image
from reportlab.lib.styles import getSampleStyleSheet
from reportlab.lib.pagesizes import A4

def main():
    import argparse
    parser = argparse.ArgumentParser(
        prog='HOS_M2F.2pdf',
        description='Markdown 转 PDF',
        epilog=(
            '示例:\n'
            '  HOS_M2F.2pdf input.md output.pdf\n'
            '相关命令:\n'
            '  HOS_M2F.2docx input.md output.docx [-p paper|patent|softcopy] [-nm] [-nc] [-q]\n'
            '  HOS_M2F.2html input.md output.html [-p wx|csdn]\n'
            '  HOS_M2F.2json input.md output.json\n'
            '  HOS_M2F.2xml  input.md output.xml\n'
            '  HOS_M2F.2png  input.mmd output.png\n'
        ),
        formatter_class=argparse.RawTextHelpFormatter,
    )
    parser.add_argument('input', help='输入 Markdown 文件路径')
    parser.add_argument('output', help='输出 PDF 文件路径')
    args = parser.parse_args()
    inp = Path(args.input)
    outp = Path(args.output)
    text = inp.read_text(encoding='utf-8')
    with tempfile.TemporaryDirectory() as td:
        model = parse_to_model(text, Path(td))
        styles = getSampleStyleSheet()
        flow = []
        doc = SimpleDocTemplate(str(outp), pagesize=A4)
        page_w, page_h = A4
        max_w = page_w - doc.leftMargin - doc.rightMargin
        for b in model['blocks']:
            t = b['type']
            if t == 'heading':
                lvl = int(b.get('level',1))
                style_name = 'Heading' + str(min(3, lvl))
                st = styles[style_name] if style_name in styles else styles['Heading1']
                flow.append(Paragraph(b.get('text',''), st))
                flow.append(Spacer(1, 8))
            elif t == 'text':
                flow.append(Paragraph(b.get('text',''), styles['BodyText']))
            elif t == 'list':
                flow.append(Paragraph('• ' + b.get('text',''), styles['BodyText']))
            elif t == 'code':
                flow.append(Paragraph('<font face="Courier">' + b.get('text','').replace('&','&amp;').replace('<','&lt;').replace('>','&gt;') + '</font>', styles['BodyText']))
            elif t == 'image':
                p = b.get('path','')
                if p:
                    img = Image(p)
                    img._restrictSize(max_w, page_h * 0.6)
                    flow.append(img)
            elif t == 'table':
                txt = '\n'.join([' | '.join(b.get('header',[]))] + [' | '.join(r) for r in b.get('rows',[])])
                flow.append(Paragraph(txt, styles['BodyText']))
        doc.build(flow)
    print(f'Converted to PDF: {inp} -> {outp}')

if __name__ == '__main__':
    main()
